# Before `make install' is performed this script should be runnable with
# `make test'. After `make install' it should work as `perl 1.t'

#########################

use Test::More ( skip_all => "Skipped until better tests are written." );

# use Test::More;
# BEGIN { use_ok( 'UI::Dialog::Backend::Nautilus' ); }
# require_ok( 'UI::Dialog::Backend::Nautilus' );

# #########################

# my $obj = UI::Dialog::Backend::Nautilus->new();
# isa_ok( $obj, 'UI::Dialog::Backend::Nautilus' );

# my @methods = qw( path paths uri uris geometry uri_unescape );
# can_ok( 'UI::Dialog::Backend::Nautilus', @methods );

# done_testing();
